﻿package  {
	import org.flixel.FlxBlock;
	import org.flixel.FlxCore;
	import org.flixel.FlxG;
	import org.flixel.FlxSprite;
	
	public class Player extends FlxSprite {
		
		[Embed(source = "../data/player2.png")] private var ImgPlayer:Class;
		
		private const _SPEED:int = 50;
		private const _GRAVITY:int = 400;
		private const _JUMPFORCE:int = 100;
		
		private const _LEFT:String = "left";
		private const _RIGHT:String = "right";
		private const _UP:String = "up";
		private const _DOWN:String = "down";
		
		private var _jumping:Boolean = true;
		private var _direction:String = _RIGHT;
		private var _wallTouch:String;
		private var _collided:String;
		
		public function Player(X:int, Y:int) {
			super(ImgPlayer, X, Y, true, true, 7, 7);
			
			maxVelocity.x = 100;
			maxVelocity.y = 100;
			acceleration.y = _GRAVITY;
			
		/*	width = 2;
			height = 6;
			offset.x = 3;
			offset.y = 2; */
			
			addAnimation("running", [0, 0, 1], 15);
			addAnimation("jumping", [2]);
			addAnimation("falling", [3]);
		}
		
		override public function update():void {
			// jump
			if ((!_jumping) &&
				( (FlxG.keys.justPressed("SPACE"))
				|| (FlxG.mouse.justPressed())
				|| (FlxG.keys.justPressed("X"))
				|| (FlxG.keys.justPressed("C"))
				|| (FlxG.keys.justPressed("UP"))
				|| (FlxG.keys.justPressed("DOWN"))
				|| (FlxG.keys.justPressed("LEFT"))
				|| (FlxG.keys.justPressed("RIGHT"))
				)) {
				if (_wallTouch != _UP) {
					if (_direction != _DOWN) velocity.y = -_JUMPFORCE;
				} else {
					velocity.y = _JUMPFORCE;
					facing = (facing == LEFT) ? RIGHT : LEFT;
				}
				//if (_direction == _DOWN) velocity.y = 0;
				
				if (_wallTouch == _RIGHT) {
					_direction = _LEFT;
					facing = LEFT;
				}
				else if (_wallTouch == _LEFT) {
					_direction = _RIGHT;
					facing = RIGHT;
				}
				
				angle = 0;
				acceleration.x = 0;
				acceleration.y = _GRAVITY;
				_wallTouch = undefined;
				_jumping = true;
			}
			
			// move with constant speed
			if (_direction == _RIGHT) velocity.x = _SPEED;
			else if (_direction == _LEFT) velocity.x = -_SPEED;
			else if (_direction == _UP) velocity.y = -_SPEED;
			else if (_direction == _DOWN) velocity.y = _SPEED;
			else trace("'_direction' error:", _direction);
			
			// debug stop
			if (FlxG.debug && (FlxG.keys.CONTROL)) {
				velocity.x = 0;
				velocity.y = 0;
			}
			
			// update
			super.update();
			
			// make sure character sticks to walls
			if ((!_jumping) && (!_collided)) {
				trace("on the air", _jumping, _collided);
				if ((_wallTouch == _DOWN) || (_wallTouch == _UP)) {
					if (_direction == _RIGHT) {
						angle = -270;
						acceleration.x = -_GRAVITY;
						if (_wallTouch == _UP) facing = LEFT;
						else facing = RIGHT;
					} else if (_direction == _LEFT) {
						angle = -90;
						acceleration.x = _GRAVITY;
						if (_wallTouch == _UP) facing = RIGHT;
						else facing = LEFT;
					}
					acceleration.y = 0;
					velocity.x = 0;
					_direction = _wallTouch;
				} else if ((_wallTouch == _RIGHT) || (_wallTouch == _LEFT)) {
					if (_direction == _UP) {
						angle = 0;
						acceleration.y = _GRAVITY;
						if (_wallTouch == _RIGHT) facing = RIGHT;
						else facing = LEFT;
					} else if (_direction == _DOWN) {
						angle = 180;
						acceleration.y = -_GRAVITY;
						if (_wallTouch == _RIGHT) facing = LEFT;
						else facing = RIGHT;
					}
					acceleration.x = 0;
					velocity.y = 0;
					_direction = _wallTouch;
				}
				_wallTouch = undefined;
				_jumping = true;
			}
			_collided = undefined;
			
			// animate
			if (_jumping) {
				if (velocity.y >= 0) play("falling");
				else play("jumping");
			}
			else play("running");
		}
		
		override public function hitFloor(Contact:FlxCore = null):Boolean {
			
			// TEST
			//trace(Contact.);
			
			//trace("HIT FLOOR");
			_collided = _DOWN;
			
			if ((_wallTouch != _DOWN) && (_direction != _UP)) {
				//trace("now running along floor, prev: touching", _wallTouch, ", going", _direction, ", jumping", _jumping);
				angle = 0;
				
				if ((_wallTouch == _LEFT) || (_jumping && (velocity.x >= 0))) {
					facing = FlxSprite.RIGHT;
					_direction = _RIGHT;
				} else {
					facing = FlxSprite.LEFT;
					_direction = _LEFT;
				}
				
				acceleration.y = _GRAVITY;
				acceleration.x = 0;
				
				play("running");
				
				_jumping = false;
				_wallTouch = _DOWN;
				//trace("   now: touching", _wallTouch, ", going", _direction, ", jumping", _jumping);
			}
			
			//return super.hitFloor(Contact);
			return true;
		}
		override public function hitCeiling(Contact:FlxCore = null):Boolean {
			_collided = _UP;
			//trace("HIT CEILING");
			
			if ((_wallTouch != _UP) && (_direction != _DOWN)) {
				//trace("now running along ceiling, prev: touching", _wallTouch, ", going", _direction, ", jumping", _jumping);
				angle = 180;
				
				if ((_wallTouch == _RIGHT) || (_jumping && (velocity.x < 0))) {
					facing = FlxSprite.RIGHT;
					_direction = _LEFT;
				} else {
					facing = FlxSprite.LEFT;
					_direction = _RIGHT;
				}
				
				acceleration.y = -_GRAVITY;
				acceleration.x = 0;
				
				play("running");
				
				_jumping = false;
				_wallTouch = _UP;
				//trace("   now: touching", _wallTouch, ", going", _direction, ", jumping", _jumping);
			}
			
			//return super.hitCeiling(Contact);
			return true;
		}
		override public function hitWall(Contact:FlxCore = null, RightHit:* = null):Boolean {
			if (RightHit) _collided = _RIGHT;
			else _collided = _LEFT;
			//trace("HIT WALL; on right?", RightHit);
			
			// RightHit "true" means that the wall is to the right of this sprite.
			
			if ( ((!RightHit) && (_wallTouch != _LEFT) && (_direction != _RIGHT))
				|| ((RightHit) && (_wallTouch != _RIGHT) && (_direction != _LEFT)) ) {
				
				trace("touched wall", RightHit, _wallTouch, _direction);
				
				if (RightHit) { // wall is on right
					if ((_wallTouch == _UP) || (_jumping && (velocity.y > 0))) {
						facing = FlxSprite.LEFT;
						_direction = _DOWN;
					} else {
						facing = FlxSprite.RIGHT;
						_direction = _UP;
					}
					angle = -90;
					acceleration.x = _GRAVITY;
					_wallTouch = _RIGHT;
				} else { // wall is on left
					if ((_wallTouch == _UP) || (_jumping && (velocity.y > 0))) {
						facing = FlxSprite.RIGHT;
						_direction = _DOWN;
					} else {
						facing = FlxSprite.LEFT;
						_direction = _UP;
					}
					angle = -270;
					acceleration.x = -_GRAVITY;
					_wallTouch = _LEFT;
				}
				
				acceleration.y = 0;
				
				play("running");
				
				_jumping = false;
			}
			
			//return super.hitWall(Contact);
			return true;
		}
		
	}
	
}